@@ -49,7 +49,7 @@ import java.util.*;
  * @author Bing Ran
  * @author <a href="mailto:scottstirling@rcn.com">Scott Stirling</a>
  * @author <a href="mailto:blackdrag@gmx.org">Jochen Theodorou</a>
- * @version $Revision: 15765 $
+ * @version $Revision: 16982 $
  */
 public class GroovyClassLoader extends URLClassLoader {
 
@@ -172,7 +172,7 @@ public class GroovyClassLoader extends URLClassLoader {
             unit.addClassNode(classNode);
             unit.setClassgenCallback(collector);
             unit.compile(Phases.CLASS_GENERATION);
-
+            definePackage(collector.generatedClass.getName());
             return collector.generatedClass;
         } catch (CompilationFailedException e) {
             throw new RuntimeException(e);
@@ -280,6 +280,7 @@ public class GroovyClassLoader extends URLClassLoader {
             answer = collector.generatedClass;
             for (Iterator iter = collector.getLoadedClasses().iterator(); iter.hasNext();) {
                 Class clazz = (Class) iter.next();
+                definePackage(clazz.getName());
                 setClassCacheEntry(clazz);
             }
             if (shouldCacheSource) sourceCache.put(codeSource.getName(), answer);
@@ -287,6 +288,17 @@ public class GroovyClassLoader extends URLClassLoader {
         }
     }
 
+    private void definePackage(String className) {
+    	int i = className.lastIndexOf('.');
+    	if (i != -1) {
+    		String pkgName = className.substring(0, i);
+    		Package pkg = getPackage(pkgName);
+    		if (pkg == null) {
+    			definePackage(pkgName, null, null, null, null, null, null, null);
+    		}
+    	}
+    }
+    	
     /**
      * gets the currently used classpath.
      *
@@ -339,8 +351,10 @@ public class GroovyClassLoader extends URLClassLoader {
             }
         });
         PermissionCollection myPerms = myDomain.getPermissions();
-        for (Enumeration<Permission> elements = myPerms.elements(); elements.hasMoreElements();) {
-            perms.add(elements.nextElement());
+        if(myPerms != null) {
+            for (Enumeration<Permission> elements = myPerms.elements(); elements.hasMoreElements();) {
+                perms.add(elements.nextElement());
+            }
         }
         perms.setReadOnly();
         return perms;
@@ -684,7 +698,10 @@ public class GroovyClassLoader extends URLClassLoader {
                     final Class classCacheEntry = getClassCacheEntry(name);
                     if (classCacheEntry != cls) return classCacheEntry;
                     URL source = resourceLoader.loadGroovySource(name);
-                    cls = recompile(source, name, cls);
+                    // if recompilation fails, we want cls==null
+                    Class oldClass = cls;
+                    cls = null;
+                    cls = recompile(source, name, oldClass);
                 } catch (IOException ioe) {
                     last = new ClassNotFoundException("IOException while opening groovy source: " + name, ioe);
                 } finally {
@@ -776,32 +793,38 @@ public class GroovyClassLoader extends URLClassLoader {
 
         return decodedFile;
     }
+    
+    private boolean isFile(URL ret) {
+         return ret != null && ret.getProtocol().equals("file");
+    }
 
+    private File getFileForUrl(URL ret, String filename) {
+        String fileWithoutPackage = filename;
+        if (fileWithoutPackage.indexOf('/') != -1) {
+            int index = fileWithoutPackage.lastIndexOf('/');
+            fileWithoutPackage = fileWithoutPackage.substring(index + 1);
+        }
+        File path = new File(decodeFileName(ret.getFile())).getParentFile();
+        if (path.exists() && path.isDirectory()) {
+            File file = new File(path, fileWithoutPackage);
+            if (file.exists()) {
+                // file.exists() might be case insensitive. Let's do
+                // case sensitive match for the filename
+                File parent = file.getParentFile();
+                String[] files = parent.list();
+                for (int j = 0; j < files.length; j++) {
+                    if (files[j].equals(fileWithoutPackage)) return file;
+                }
+            }
+        }
+        //file does not exist!
+        return null;
+    }
+    
     private URL getSourceFile(String name) {
         String filename = name.replace('.', '/') + config.getDefaultScriptExtension();
         URL ret = getResource(filename);
-        if (ret != null && ret.getProtocol().equals("file")) {
-            String fileWithoutPackage = filename;
-            if (fileWithoutPackage.indexOf('/') != -1) {
-                int index = fileWithoutPackage.lastIndexOf('/');
-                fileWithoutPackage = fileWithoutPackage.substring(index + 1);
-            }
-            File path = new File(decodeFileName(ret.getFile())).getParentFile();
-            if (path.exists() && path.isDirectory()) {
-                File file = new File(path, fileWithoutPackage);
-                if (file.exists()) {
-                    // file.exists() might be case insensitive. Let's do
-                    // case sensitive match for the filename
-                    File parent = file.getParentFile();
-                    String[] files = parent.list();
-                    for (int j = 0; j < files.length; j++) {
-                        if (files[j].equals(fileWithoutPackage)) return ret;
-                    }
-                }
-            }
-            //file does not exist!
-            return null;
-        }
+        if (isFile(ret) && getFileForUrl(ret,filename)==null) return null;
         return ret;
     }
 
@@ -820,7 +843,7 @@ public class GroovyClassLoader extends URLClassLoader {
 
         // Special handling for file:// protocol, as getLastModified() often reports
         // incorrect results (-1)
-        if (source.getProtocol().equals("file")) {
+        if (isFile(source)) {
             // Coerce the file URL to a File
             String path = source.getPath().replace('/', File.separatorChar).replace('|', ':');
             File file = new File(path);
