@@ -40,13 +40,25 @@ import javax.sql.DataSource;
 import org.codehaus.groovy.runtime.DefaultGroovyMethods;
 
 /**
- * Represents an extent of objects
+ * A facade over Java's normal JDBC apis providing greatly simplified
+ * resource management and result set handling. Under the covers the
+ * facade hides away details associated with getting connections,
+ * constructing and configuring statements, interacting with the
+ * connection, closing resources and logging errors. Special
+ * features of the facade include using closures to iterate
+ * through result sets, a special GString syntax for representing
+ * prepared statements and treating result sets like collections
+ * of maps with the normal Groovy collection methods available.
+ *
+ * The class provides numerous extension points for overriding the
+ * facade behavior associated with the various aspects of managing
+ * the connection with the underlying database.
  *
  * @author Chris Stevenson
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
  * @author Paul King
  * @author Marc DeXeT
- * @version $Revision: 16362 $
+ * @version $Revision: 17066 $
  */
 public class Sql {
 
@@ -733,7 +745,7 @@ public class Sql {
      * Performs the given SQL query and return the first row of the result set.
      *
      * @param sql the SQL statement
-     * @return a GroovyRowResult object
+     * @return a GroovyRowResult object or <code>null</code> if no row is found
      * @throws SQLException if a database access error occurs
      */
     public Object firstRow(String sql) throws SQLException {
@@ -747,7 +759,7 @@ public class Sql {
      * the first row of the result set.
      *
      * @param gstring a GString containing the SQL query with embedded params
-     * @return a GroovyRowResult object
+     * @return a GroovyRowResult object or <code>null</code> if no row is found
      * @throws SQLException if a database access error occurs
      */
     public Object firstRow(GString gstring) throws SQLException {
@@ -762,7 +774,7 @@ public class Sql {
      *
      * @param sql    the SQL statement
      * @param params a list of parameters
-     * @return a GroovyRowResult object
+     * @return a GroovyRowResult object or <code>null</code> if no row is found
      * @throws SQLException if a database access error occurs
      */
     public Object firstRow(String sql, List params) throws SQLException {
@@ -1641,6 +1653,60 @@ public class Sql {
         }
     }
 
+    /**
+     * Performs the closure within a batch using a cached connection.
+     * The closure will be called with a single argument; the statement
+     * associated with this batch. Use it like this:
+     * <pre>
+     * def updateCounts = sql.withBatch { stmt ->
+     *     stmt.addBatch("insert into TABLENAME ...")
+     *     stmt.addBatch("insert into TABLENAME ...")
+     *     stmt.addBatch("insert into TABLENAME ...")
+     * }
+     * </pre>
+     *
+     * @param closure the closure containing batch and optionally other statements
+     * @return an array of update counts containing one element for each
+     * command in the batch.  The elements of the array are ordered according
+     * to the order in which commands were added to the batch.
+     * @exception SQLException if a database access error occurs,
+     * or this method is called on a closed <code>Statement</code>, or the
+     * driver does not support batch statements. Throws {@link java.sql.BatchUpdateException}
+     * (a subclass of <code>SQLException</code>) if one of the commands sent to the
+     * database fails to execute properly or attempts to return a result set.
+     */
+    public synchronized int[] withBatch(Closure closure) throws SQLException {
+        boolean savedCacheConnection = cacheConnection;
+        cacheConnection = true;
+        Connection connection = null;
+        try {
+            connection = createConnection();
+            connection.setAutoCommit(false);
+            Statement statement = connection.createStatement();
+            closure.call(statement);
+            int[] result = statement.executeBatch();
+            connection.commit();
+            return result;
+        } catch (SQLException e) {
+            handleError(connection, e);
+            throw e;
+        } catch (RuntimeException e) {
+            handleError(connection, e);
+            throw e;
+        } catch (Error e) {
+            handleError(connection, e);
+            throw e;
+        } finally {
+            if (connection != null) connection.setAutoCommit(true);
+            cacheConnection = false;
+            closeResources(connection, null);
+            cacheConnection = savedCacheConnection;
+            if (dataSource != null && !cacheConnection) {
+                useConnection = null;
+            }
+        }
+    }
+
     private void handleError(Connection connection, Throwable t) throws SQLException {
         if (connection != null) {
             log.log(Level.INFO, "Rolling back due to: " + t.getMessage(), t);
