@@ -23,9 +23,9 @@ import org.codehaus.groovy.runtime.typehandling.DefaultTypeTransformation;
  * A Comparator which uses a closure to compare 2 values being equal
  * 
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @version $Revision: 6778 $
+ * @version $Revision: 16878 $
  */
-public class ClosureComparator implements Comparator {
+public class ClosureComparator<T> implements Comparator<T> {
 
     Closure closure;
 
@@ -33,7 +33,7 @@ public class ClosureComparator implements Comparator {
         this.closure = closure;
     }
 
-    public int compare(Object object1, Object object2) {
+    public int compare(T object1, T object2) {
         Object value = closure.call(new Object[] {object1, object2});
         return DefaultTypeTransformation.intUnbox(value);
     }
