@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ * Copyright 2003-2008 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,6 +17,7 @@ package groovy.util;
 
 import groovy.lang.*;
 import org.codehaus.groovy.control.MultipleCompilationErrorsException;
+import org.codehaus.groovy.runtime.ConversionHandler;
 import org.codehaus.groovy.runtime.DefaultGroovyMethods;
 import org.codehaus.groovy.runtime.InvokerHelper;
 
@@ -25,8 +26,8 @@ import java.lang.reflect.Modifier;
 import java.util.*;
 
 /**
- * Classes to generate 'Proxy' objects which implement interfaces
- * and/or extend classes.
+ * Classes to generate 'Proxy' objects which implement interfaces,
+ * maps of closures and/or extend classes/delegates.
  *
  * @author Paul King
  * @author Guillaume Laforge
@@ -41,15 +42,43 @@ public class ProxyGenerator {
 
     private ClassLoader override = null;
     private boolean debug = false;
+    private boolean emptyMethods = false;
+    private List objectMethods = getInheritedMethods(Object.class, new ArrayList());
+    private List groovyObjectMethods = getInheritedMethods(GroovyObject.class, new ArrayList());
 
     public boolean getDebug() {
         return debug;
     }
 
+    /**
+     * Instructs <code>ProxyGenerator</code> to dump generated Groovy
+     * source code to standard output during construction. This is useful
+     * for debugging purposes but should be turned off in production.
+     *
+     * @param debug true if you want generated source to be printed
+     */
     public void setDebug(boolean debug) {
         this.debug = debug;
     }
 
+    public boolean getEmptyMethods() {
+        return emptyMethods;
+    }
+
+    /**
+     * Changes generated methods to have empty implementations.
+     *
+     * Methods in generated aggregates not supplied in a closures map or
+     * base class are given 'default' implementations. The implementation
+     * will normally throw an <code>UnsupportedOperationException</code>
+     * but setting this boolean will leave it empty.
+     *
+     * @param emptyMethods true if you want generated methods to be empty
+     */
+    public void setEmptyMethods(boolean emptyMethods) {
+        this.emptyMethods = emptyMethods;
+    }
+
     public ClassLoader getOverride() {
         return override;
     }
@@ -59,13 +88,19 @@ public class ProxyGenerator {
     }
 
     public Object instantiateAggregateFromBaseClass(Class clazz) {
-        return instantiateAggregateFromBaseClass(null, clazz);
+        return instantiateAggregateFromBaseClass((Map)null, clazz);
     }
 
     public Object instantiateAggregateFromBaseClass(Map map, Class clazz) {
         return instantiateAggregateFromBaseClass(map, clazz, null);
     }
 
+    public Object instantiateAggregateFromBaseClass(Closure cl, Class clazz) {
+        Map m = new HashMap();
+        m.put("*", cl);
+        return instantiateAggregateFromBaseClass(m, clazz, null);
+    }
+
     public Object instantiateAggregateFromBaseClass(Map map, Class clazz, Object[] constructorArgs) {
         return instantiateAggregate(map, null, clazz, constructorArgs);
     }
@@ -140,16 +175,18 @@ public class ProxyGenerator {
         buffer.append("    }\n");
 
         // add overwriting methods
-        List selectedMethods = new ArrayList();
-        List publicAndProtectedMethods = DefaultGroovyMethods.toList(baseClass.getMethods());
-        publicAndProtectedMethods.addAll(getInheritedMethods(baseClass));
-        for (int i = 0; i < publicAndProtectedMethods.size(); i++) {
-            Method method = (Method) publicAndProtectedMethods.get(i);
-            if (method.getName().indexOf('$') != -1)
-              continue;
-            if (map.containsKey(method.getName())) {
-                selectedMethods.add(method.getName());
-                addOverridingMapCall(buffer, method);
+        Map<String, Method> selectedMethods = new HashMap<String, Method>();
+        List<Method> publicAndProtectedMethods = getInheritedMethods(baseClass, new ArrayList<Method>());
+        boolean closureIndicator = map.containsKey("*");
+        for (Method method : publicAndProtectedMethods) {
+            if (method.getName().indexOf('$') != -1
+                    || Modifier.isFinal(method.getModifiers())
+                    || ConversionHandler.isCoreObjectMethod(method)
+                    || containsEquivalentMethod(selectedMethods.values(), method))
+                continue;
+            if (map.containsKey(method.getName()) || closureIndicator) {
+                selectedMethods.put(method.getName(), method);
+                addOverridingMapCall(buffer, method, closureIndicator);
             }
         }
 
@@ -157,13 +194,12 @@ public class ProxyGenerator {
         List interfaceMethods = new ArrayList();
         for (int i = 0; i < interfacesToImplement.size(); i++) {
             Class thisInterface = (Class) interfacesToImplement.get(i);
-            interfaceMethods.addAll(DefaultGroovyMethods.toList(thisInterface.getMethods()));
-            interfaceMethods.addAll(getInheritedMethods(thisInterface));
+            getInheritedMethods(thisInterface, interfaceMethods);
         }
         for (int i = 0; i < interfaceMethods.size(); i++) {
             Method method = (Method) interfaceMethods.get(i);
             if (!containsEquivalentMethod(publicAndProtectedMethods, method)) {
-                selectedMethods.add(method.getName());
+                selectedMethods.put(method.getName(), method);
                 addMapOrDummyCall(map, buffer, method);
             }
         }
@@ -171,9 +207,9 @@ public class ProxyGenerator {
         // add leftover methods from the map
         for (Iterator iterator = map.keySet().iterator(); iterator.hasNext();) {
             String methodName = (String) iterator.next();
-            if (methodName.indexOf('$') != -1)
-              continue;
-            if (selectedMethods.contains(methodName)) continue;
+            if (methodName.indexOf('$') != -1 || methodName.indexOf('*') != -1)
+                continue;
+            if (selectedMethods.keySet().contains(methodName)) continue;
             addNewMapCall(buffer, methodName);
         }
 
@@ -242,7 +278,7 @@ public class ProxyGenerator {
         if (baseClass != null) {
             buffer.append(" extends ").append(baseClass.getName());
         }
-        
+
         for (int i = 0; i < interfacesToImplement.size(); i++) {
             Class thisInterface = (Class) interfacesToImplement.get(i);
             if (i == 0) {
@@ -260,18 +296,11 @@ public class ProxyGenerator {
         buffer.append("        this.delegate = delegate\n");
         buffer.append("    }\n");
 
-        List objectMethods = DefaultGroovyMethods.toList(Object.class.getMethods());
-        objectMethods.addAll(getInheritedMethods(Object.class));
-
-        List groovyObjectMethods = DefaultGroovyMethods.toList(GroovyObject.class.getMethods());
-        groovyObjectMethods.addAll(getInheritedMethods(GroovyObject.class));
-
         // add interface methods
         List interfaceMethods = new ArrayList();
         for (int i = 0; i < interfacesToImplement.size(); i++) {
             Class thisInterface = (Class) interfacesToImplement.get(i);
-            interfaceMethods.addAll(DefaultGroovyMethods.toList(thisInterface.getMethods()));
-            interfaceMethods.addAll(getInheritedMethods(thisInterface));
+            getInheritedMethods(thisInterface, interfaceMethods);
         }
         for (int i = 0; i < interfaceMethods.size(); i++) {
             Method method = (Method) interfaceMethods.get(i);
@@ -281,13 +310,11 @@ public class ProxyGenerator {
                 addWrappedCall(buffer, method, map);
             }
         }
-        List additionalMethods = new ArrayList();
-        additionalMethods.addAll(DefaultGroovyMethods.toList(delegate.getClass().getMethods()));
-        additionalMethods.addAll(getInheritedMethods(delegate.getClass()));
+        List additionalMethods = getInheritedMethods(delegate.getClass(), new ArrayList());
         for (int i = 0; i < additionalMethods.size(); i++) {
             Method method = (Method) additionalMethods.get(i);
             if (method.getName().indexOf('$') != -1)
-              continue;
+                continue;
             if (!containsEquivalentMethod(interfaceMethods, method) &&
                     !containsEquivalentMethod(objectMethods, method) &&
                     !containsEquivalentMethod(groovyObjectMethods, method)) {
@@ -324,7 +351,7 @@ public class ProxyGenerator {
 
     private void addWrappedCall(StringBuffer buffer, Method method, Map map) {
         if (map.containsKey(method.getName())) {
-            addOverridingMapCall(buffer, method);
+            addOverridingMapCall(buffer, method, false);
         } else {
             Class[] parameterTypes = addMethodPrefix(buffer, method);
             addWrappedMethodBody(buffer, method, parameterTypes);
@@ -332,9 +359,8 @@ public class ProxyGenerator {
         }
     }
 
-    private boolean containsEquivalentMethod(List publicAndProtectedMethods, Method candidate) {
-        for (int i = 0; i < publicAndProtectedMethods.size(); i++) {
-            Method method = (Method) publicAndProtectedMethods.get(i);
+    private boolean containsEquivalentMethod(Collection<Method> publicAndProtectedMethods, Method candidate) {
+        for (Method method : publicAndProtectedMethods) {
             if (candidate.getName().equals(method.getName()) &&
                     candidate.getReturnType().equals(method.getReturnType()) &&
                     hasMatchingParameterTypes(candidate, method)) {
@@ -354,21 +380,21 @@ public class ProxyGenerator {
         return true;
     }
 
-    private List getInheritedMethods(Class baseClass) {
-        List protectedMethodList = new ArrayList();
+    private List<Method> getInheritedMethods(Class baseClass, List<Method> methods) {
+        methods.addAll(DefaultGroovyMethods.toList(baseClass.getMethods()));
         Class currentClass = baseClass;
         while (currentClass != null) {
             Method[] protectedMethods = currentClass.getDeclaredMethods();
             for (int i = 0; i < protectedMethods.length; i++) {
                 Method method = protectedMethods[i];
                 if (method.getName().indexOf('$') != -1)
-                  continue;
-                if (Modifier.isProtected(method.getModifiers()) && !containsEquivalentMethod(protectedMethodList, method))
-                    protectedMethodList.add(method);
+                    continue;
+                if (Modifier.isProtected(method.getModifiers()) && !containsEquivalentMethod(methods, method))
+                    methods.add(method);
             }
             currentClass = currentClass.getSuperclass();
         }
-        return protectedMethodList;
+        return methods;
     }
 
     private void addNewMapCall(StringBuffer buffer, String methodName) {
@@ -376,20 +402,26 @@ public class ProxyGenerator {
                 .append("        this.@closureMap['").append(methodName).append("'] (*args)\n    }\n");
     }
 
-    private void addOverridingMapCall(StringBuffer buffer, Method method) {
+    private void addOverridingMapCall(StringBuffer buffer, Method method, boolean closureIndicator) {
         Class[] parameterTypes = addMethodPrefix(buffer, method);
-        addMethodBody(buffer, method, parameterTypes);
+        addMethodBody(buffer, closureIndicator ? "*" : method.getName(), parameterTypes);
         addMethodSuffix(buffer);
     }
 
     private void addMapOrDummyCall(Map map, StringBuffer buffer, Method method) {
         Class[] parameterTypes = addMethodPrefix(buffer, method);
         if (map.containsKey(method.getName())) {
-            addMethodBody(buffer, method, parameterTypes);
+            addMethodBody(buffer, method.getName(), parameterTypes);
+        } else if (!emptyMethods) {
+            addUnsupportedBody(buffer);
         }
         addMethodSuffix(buffer);
     }
 
+    private void addUnsupportedBody(StringBuffer buffer) {
+        buffer.append("throw new UnsupportedOperationException()");
+    }
+
     private Class[] addMethodPrefix(StringBuffer buffer, Method method) {
         buffer.append("    ").append(getSimpleName(method.getReturnType()))
                 .append(" ").append(method.getName()).append("(");
@@ -406,8 +438,8 @@ public class ProxyGenerator {
         return parameterTypes;
     }
 
-    private void addMethodBody(StringBuffer buffer, Method method, Class[] parameterTypes) {
-        buffer.append("this.@closureMap['").append(method.getName()).append("'] (");
+    private void addMethodBody(StringBuffer buffer, String method, Class[] parameterTypes) {
+        buffer.append("this.@closureMap['").append(method).append("'] (");
         for (int j = 0; j < parameterTypes.length; j++) {
             if (j != 0) {
                 buffer.append(", ");
@@ -448,7 +480,7 @@ public class ProxyGenerator {
                 dimension++;
             }
             return componentClass.getName().replaceAll("\\$", "\\.") +
-                    DefaultGroovyMethods.multiply("[]", new Integer(dimension));
+                    DefaultGroovyMethods.multiply("[]", Integer.valueOf(dimension));
         } else {
             return c.getName().replaceAll("\\$", "\\.");
         }
@@ -462,9 +494,7 @@ public class ProxyGenerator {
 
     private static void setMetaClass(final MetaClass metaClass) {
         final MetaClass newMetaClass = new DelegatingMetaClass(metaClass) {
-            /* (non-Javadoc)
-            * @see groovy.lang.MetaClass#invokeStaticMethod(java.lang.Object, java.lang.String, java.lang.Object[])
-            */
+            @Override
             public Object invokeStaticMethod(Object object, String methodName, Object[] arguments) {
                 return InvokerHelper.invokeMethod(INSTANCE, methodName, arguments);
             }
