@@ -42,7 +42,7 @@ import org.codehaus.groovy.control.messages.SyntaxErrorMessage;
 import org.codehaus.groovy.syntax.SyntaxException;
 
 public abstract class ClassCodeVisitorSupport extends CodeVisitorSupport implements GroovyClassVisitor {
-    
+
     public void visitClass(ClassNode node) {
         visitAnnotations(node);
         node.visitContents(this);
@@ -52,23 +52,23 @@ public abstract class ClassCodeVisitorSupport extends CodeVisitorSupport impleme
             element.visit(this);
         }
     }
-    
+
     public void visitAnnotations(AnnotatedNode node) {
-        List annotions = node.getAnnotations();
-        if (annotions.isEmpty()) return;
-        Iterator it = annotions.iterator();
+        List annotations = node.getAnnotations();
+        if (annotations.isEmpty()) return;
+        Iterator it = annotations.iterator();
         while (it.hasNext()) {
             AnnotationNode an = (AnnotationNode) it.next();
-            //skip builtin properties
+            // skip built-in properties
             if (an.isBuiltIn()) continue;
             for (Iterator iter = an.getMembers().entrySet().iterator(); iter.hasNext();) {
                 Map.Entry member = (Map.Entry) iter.next();
                 Expression memberValue = (Expression) member.getValue();
                 memberValue.visit(this);
-            }  
+            }
         }
     }
-        
+
     protected void visitClassCodeContainer(Statement code) {
         if (code != null) code.visit(this);
     }
@@ -76,12 +76,12 @@ public abstract class ClassCodeVisitorSupport extends CodeVisitorSupport impleme
     protected void visitConstructorOrMethod(MethodNode node, boolean isConstructor) {
         visitAnnotations(node);
         Statement code = node.getCode();
-        
+
         visitClassCodeContainer(code);
     }
-    
+
     public void visitConstructor(ConstructorNode node) {
-        visitConstructorOrMethod(node,true);        
+        visitConstructorOrMethod(node,true);
     }
 
     public void visitMethod(MethodNode node) {
@@ -93,15 +93,15 @@ public abstract class ClassCodeVisitorSupport extends CodeVisitorSupport impleme
         Expression init = node.getInitialExpression();
         if (init != null) init.visit(this);
     }
-    
+
     public void visitProperty(PropertyNode node) {
         visitAnnotations(node);
         Statement statement = node.getGetterBlock();
         visitClassCodeContainer(statement);
-        
+
         statement = node.getSetterBlock();
         visitClassCodeContainer(statement);
-        
+
         Expression init = node.getInitialExpression();
         if (init != null) init.visit(this);
     }
@@ -114,86 +114,86 @@ public abstract class ClassCodeVisitorSupport extends CodeVisitorSupport impleme
           new SyntaxErrorMessage(new SyntaxException(msg + '\n', line, col), source)
         );
     }
-    
+
     protected abstract SourceUnit getSourceUnit();
-    
+
     protected void visitStatement(Statement statement) {}
-    
+
     public void visitAssertStatement(AssertStatement statement) {
         visitStatement(statement);
         super.visitAssertStatement(statement);
     }
-    
+
     public void visitBlockStatement(BlockStatement block) {
         visitStatement(block);
         super.visitBlockStatement(block);
     }
-    
+
     public void visitBreakStatement(BreakStatement statement) {
         visitStatement(statement);
         super.visitBreakStatement(statement);
     }
-    
+
     public void visitCaseStatement(CaseStatement statement) {
         visitStatement(statement);
         super.visitCaseStatement(statement);
     }
-    
+
     public void visitCatchStatement(CatchStatement statement) {
         visitStatement(statement);
         super.visitCatchStatement(statement);
     }
-    
+
     public void visitContinueStatement(ContinueStatement statement) {
         visitStatement(statement);
         super.visitContinueStatement(statement);
     }
-    
+
     public void visitDoWhileLoop(DoWhileStatement loop) {
         visitStatement(loop);
         super.visitDoWhileLoop(loop);
     }
-    
+
     public void visitExpressionStatement(ExpressionStatement statement) {
         visitStatement(statement);
         super.visitExpressionStatement(statement);
     }
-    
+
     public void visitForLoop(ForStatement forLoop) {
         visitStatement(forLoop);
         super.visitForLoop(forLoop);
     }
-    
+
     public void visitIfElse(IfStatement ifElse) {
         visitStatement(ifElse);
         super.visitIfElse(ifElse);
     }
-    
+
     public void visitReturnStatement(ReturnStatement statement) {
         visitStatement(statement);
         super.visitReturnStatement(statement);
     }
-    
+
     public void visitSwitch(SwitchStatement statement) {
         visitStatement(statement);
         super.visitSwitch(statement);
     }
-    
+
     public void visitSynchronizedStatement(SynchronizedStatement statement) {
         visitStatement(statement);
         super.visitSynchronizedStatement(statement);
     }
-    
+
     public void visitThrowStatement(ThrowStatement statement) {
         visitStatement(statement);
         super.visitThrowStatement(statement);
     }
-    
+
     public void visitTryCatchFinally(TryCatchStatement statement) {
         visitStatement(statement);
         super.visitTryCatchFinally(statement);
     }
-    
+
     public void visitWhileLoop(WhileStatement loop) {
         visitStatement(loop);
         super.visitWhileLoop(loop);
