@@ -16,12 +16,10 @@
 
 package org.codehaus.groovy.ast;
 
-import groovy.lang.Closure;
-import groovy.lang.GString;
-import groovy.lang.MetaClass;
-import groovy.lang.Range;
-import groovy.lang.Reference;
-import groovy.lang.Script;
+import groovy.lang.*;
+import org.codehaus.groovy.runtime.GeneratedClosure;
+import org.codehaus.groovy.vmplugin.VMPluginFactory;
+import org.objectweb.asm.Opcodes;
 
 import java.math.BigDecimal;
 import java.math.BigInteger;
@@ -29,10 +27,6 @@ import java.util.List;
 import java.util.Map;
 import java.util.regex.Pattern;
 
-import org.codehaus.groovy.runtime.GeneratedClosure;
-import org.codehaus.groovy.vmplugin.VMPluginFactory;
-import org.objectweb.asm.Opcodes;
-
 /**
  * This class is a Helper for ClassNode and classes handling ClassNodes.
  * It does contain a set of predefined ClassNodes for the most used 
@@ -62,7 +56,7 @@ public class ClassHelper {
         DYNAMIC_TYPE = new ClassNode(Object.class),  OBJECT_TYPE = DYNAMIC_TYPE,
         VOID_TYPE = new ClassNode(Void.TYPE),        CLOSURE_TYPE = new ClassNode(Closure.class),
         GSTRING_TYPE = new ClassNode(GString.class), LIST_TYPE = makeWithoutCaching(List.class),
-        MAP_TYPE = new ClassNode(Map.class),         RANGE_TYPE = new ClassNode(Range.class),
+        MAP_TYPE = makeWithoutCaching(Map.class),         RANGE_TYPE = new ClassNode(Range.class),
         PATTERN_TYPE = new ClassNode(Pattern.class), STRING_TYPE = new ClassNode(String.class),
         SCRIPT_TYPE = new ClassNode(Script.class),   REFERENCE_TYPE = makeWithoutCaching(Reference.class),
         
@@ -81,10 +75,11 @@ public class ClassHelper {
         CLASS_Type = new ClassNode(Class.class),        METACLASS_TYPE = new ClassNode(MetaClass.class),
         GENERATED_CLOSURE_Type = new ClassNode(GeneratedClosure.class),
         Enum_Type = new ClassNode("java.lang.Enum",0,OBJECT_TYPE),
-        Annotation_Type = new ClassNode("java.lang.annotation.Annotation",0,OBJECT_TYPE);
-
+        Annotation_TYPE = new ClassNode("java.lang.annotation.Annotation",0,OBJECT_TYPE);
+        
     static {
         Enum_Type.isPrimaryNode = false;
+        Annotation_TYPE.isPrimaryNode = false;
     }
     
     private static ClassNode[] types = new ClassNode[] {
@@ -97,7 +92,7 @@ public class ClassHelper {
         Byte_TYPE, Short_TYPE, Integer_TYPE, Long_TYPE,
         Double_TYPE, Float_TYPE, BigDecimal_TYPE, BigInteger_TYPE, 
         void_WRAPPER_TYPE, REFERENCE_TYPE, CLASS_Type, METACLASS_TYPE,
-        GENERATED_CLOSURE_Type, Enum_Type
+        GENERATED_CLOSURE_Type, Enum_Type, Annotation_TYPE
     };
 
     
@@ -159,7 +154,7 @@ public class ClassHelper {
     
     public static ClassNode makeWithoutCaching(Class c, boolean includeGenerics){
         ClassNode t = new ClassNode(c);
-        if (includeGenerics) VMPluginFactory.getPlugin().setGenericsTypes(t);
+        if (includeGenerics) VMPluginFactory.getPlugin().setAdditionalClassInformation(t);
         return t;
     }
     
@@ -169,7 +164,7 @@ public class ClassHelper {
      * Unlike make(String) this method will not use the cache
      * to create the ClassNode. This means the ClassNode created
      * from this method using the same name will have a different
-     * references
+     * reference
      * 
      * @see #make(String)
      * @param name of the class the ClassNode is representing
@@ -184,7 +179,7 @@ public class ClassHelper {
      * Creates a ClassNode using a given class.
      * If the name is one of the predefined ClassNodes then the 
      * corresponding ClassNode instance will be returned. If the
-     * is null of of length 0 the dynamic type is returned
+     * name is null or of length 0 the dynamic type is returned
      * 
      * @param name of the class the ClassNode is representing
      */
@@ -245,7 +240,33 @@ public class ClassHelper {
             return cn;
         }
     }
-    
+
+    public static ClassNode getUnwrapper(ClassNode cn) {
+        cn = cn.redirect();
+        if (isPrimitiveType(cn)) return cn;
+        if (cn==Boolean_TYPE) {
+            return boolean_TYPE;
+        } else if (cn==Byte_TYPE) {
+            return byte_TYPE;
+        } else if (cn==Character_TYPE) {
+            return char_TYPE;
+        } else if (cn==Short_TYPE) {
+            return short_TYPE;
+        } else if (cn==Integer_TYPE) {
+            return int_TYPE;
+        } else if (cn==Long_TYPE) {
+            return long_TYPE;
+        } else if (cn==Float_TYPE) {
+            return float_TYPE;
+        } else if (cn==Double_TYPE) {
+            return double_TYPE;
+        }
+        else {
+            return cn;
+        }
+    }
+
+
     /**
      * Test to determine if a ClasNode is a primitve type. 
      * Note: this only works for ClassNodes created using a
@@ -268,16 +289,23 @@ public class ClassHelper {
                 cn == VOID_TYPE;
     }
 
-    public static ClassNode makeReference() {
-        return make(Reference.class);
+    public static boolean isNumberType(ClassNode cn) {
+        return  cn == Byte_TYPE ||
+                cn == Short_TYPE ||
+                cn == Integer_TYPE ||
+                cn == Long_TYPE ||
+                cn == Float_TYPE ||
+                cn == Double_TYPE ||
+                cn == byte_TYPE ||
+                cn == short_TYPE ||
+                cn == int_TYPE ||
+                cn == long_TYPE ||
+                cn == float_TYPE ||
+                cn == double_TYPE;
     }
 
-    public static boolean isUnresolvedEnum(ClassNode node) {
-        if (Enum_Type.isResolved()) return false;
-        if (node.isResolved()) return false;
-        ClassNode superClass = node.getSuperClass();
-        if (superClass==null) return false;
-        return superClass.redirect()==Enum_Type;
+    public static ClassNode makeReference() {
+        return make(Reference.class);
     }
 
     public static boolean isCachedType(ClassNode type) {
@@ -286,5 +314,4 @@ public class ClassHelper {
         }
         return false;
     }
-
 }
