@@ -40,7 +40,7 @@ import java.util.*;
  * bytecode generation occurs.
  *
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @version $Revision: 16221 $
+ * @version $Revision: 16690 $
  */
 public class Verifier implements GroovyClassVisitor, Opcodes {
 
@@ -93,21 +93,22 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
         metaClassField.setSynthetic(true);
         return metaClassField;
     }
-    
+
     private FieldNode getMetaClassField(ClassNode node) {
         FieldNode ret = node.getDeclaredField("metaClass");
-        if (ret!=null) return ret;
+        if (ret != null) return ret;
         ClassNode current = node;
-        while (current!=null && current!=ClassHelper.OBJECT_TYPE) {
+        while (current != ClassHelper.OBJECT_TYPE) {
             current = current.getSuperClass();
+            if (current == null) break;
             ret = current.getDeclaredField("metaClass");
-            if (ret==null) continue;
+            if (ret == null) continue;
             if (Modifier.isPrivate(ret.getModifiers())) continue;
             return ret;
         }
         return null;
     }
-    
+
     /**
      * add code to implement GroovyObject
      * @param node
@@ -431,7 +432,7 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
         if (statement!=null) statement.visit(new VerifierCodeVisitor(this));
     }
 
-    private void addReturnIfNeeded(MethodNode node) {
+    protected void addReturnIfNeeded(MethodNode node) {
         Statement statement = node.getCode();
         if (!node.isVoidMethod()) {
             if (statement != null) // it happens with @interface methods
@@ -469,6 +470,7 @@ public class Verifier implements GroovyClassVisitor, Opcodes {
             Expression expr = expStmt.getExpression(); 
             ReturnStatement ret = new ReturnStatement(expr);
             ret.setSourcePosition(expr);
+            ret.setStatementLabel(statement.getStatementLabel());
             return ret;
         }
 
