@@ -35,7 +35,7 @@ import java.util.regex.Pattern;
  * A static helper class to make bytecode generation easier and act as a facade over the Invoker
  *
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @version $Revision: 16276 $
+ * @version $Revision: 16878 $
  */
 public class InvokerHelper {
     private   static final Object[] EMPTY_MAIN_ARGS = new Object[]{new String[0]};
@@ -489,7 +489,8 @@ public class InvokerHelper {
         }
     }
 
-    public static Iterator asIterator(Object o) {
+    @SuppressWarnings("unchecked")
+    public static Iterator<Object> asIterator(Object o) {
         return (Iterator) invokeMethod(o, "iterator", EMPTY_ARGS);
     }
 
