@@ -30,14 +30,14 @@ import java.util.*;
 import java.util.regex.Matcher;
 
 public class DefaultTypeTransformation {
-
+    
     protected static final Object[] EMPTY_ARGUMENTS = {};
     protected static final BigInteger ONE_NEG = new BigInteger("-1");
-
+    
     //  --------------------------------------------------------
     //                  unboxing methods
-    //  --------------------------------------------------------
-
+    //  --------------------------------------------------------       
+    
     public static byte byteUnbox(Object value) {
         Number n = castToNumber(value);
         return n.byteValue();
@@ -74,12 +74,12 @@ public class DefaultTypeTransformation {
     public static double doubleUnbox(Object value) {
         Number n = castToNumber(value);
         return n.doubleValue();
-    }
+    } 
 
     //  --------------------------------------------------------
     //                  boxing methods
-    //  --------------------------------------------------------
-
+    //  --------------------------------------------------------       
+    
     public static Object box(boolean value) {
         return value ? Boolean.TRUE : Boolean.FALSE;
     }
@@ -97,7 +97,7 @@ public class DefaultTypeTransformation {
     }
 
     public static Object box(int value) {
-        return IntegerCache.integerValue(value);
+        return Integer.valueOf(value);
     }
 
     public static Object box(long value) {
@@ -111,25 +111,30 @@ public class DefaultTypeTransformation {
     public static Object box(double value) {
         return new Double(value);
     }
-
+    
     public static Number castToNumber(Object object) {
+    	// default to Number class in exception details, else use the specified Number subtype.
+    	return castToNumber(object, Number.class);
+    }
+    
+    public static Number castToNumber(Object object, Class type) {
         if (object instanceof Number)
             return (Number) object;
         if (object instanceof Character) {
-            return new Integer(((Character) object).charValue());
+            return Integer.valueOf(((Character) object).charValue());
         }
         if (object instanceof String) {
             String c = (String) object;
             if (c.length() == 1) {
-                return new Integer(c.charAt(0));
+                return Integer.valueOf(c.charAt(0));
             }
             else {
-                throw new GroovyCastException(c,Integer.class);
+                throw new GroovyCastException(c, type);
             }
         }
-        throw new GroovyCastException(object,Number.class);
+        throw new GroovyCastException(object, type);
     }
-
+    
     public static boolean castToBoolean(Object object) {
     	if (object == null) {
     		return false;
@@ -162,11 +167,11 @@ public class DefaultTypeTransformation {
         if (object instanceof CharSequence) {
         	CharSequence string =  (CharSequence) object;
             return string.length() > 0;
-        }
+        } 
         if (object instanceof Object[]) {
         	Object[] array =  (Object[]) object;
             return array.length > 0;
-        }
+        } 
         if (object instanceof Character) {
             Character c = (Character) object;
             return c.charValue() != 0;
@@ -177,10 +182,10 @@ public class DefaultTypeTransformation {
         }
         return true;
     }
-
+    
     public static char castToChar(Object object) {
         if (object instanceof Character) {
-            return ((Character) object).charValue();
+            return ((Character) object).charValue();            
         } else if (object instanceof Number) {
             Number value = (Number) object;
             return (char) value.intValue();
@@ -194,7 +199,7 @@ public class DefaultTypeTransformation {
             }
         }
     }
-
+    
     public static Object castToType(Object object, Class type) {
         if (object == null) {
             return null;
@@ -253,7 +258,7 @@ public class DefaultTypeTransformation {
         } else if (type == Class.class) {
             return castToClass(object);
         } else if (Number.class.isAssignableFrom(type)) {
-            Number n = castToNumber(object);
+            Number n = castToNumber(object, type);
             if (type == Byte.class) {
                 return new Byte(n.byteValue());
             } else if (type == Character.class) {
@@ -261,7 +266,7 @@ public class DefaultTypeTransformation {
             } else if (type == Short.class) {
                 return new Short(n.shortValue());
             } else if (type == Integer.class) {
-                return new Integer(n.intValue());
+                return Integer.valueOf(n.intValue());
             } else if (type == Long.class) {
                 return new Long(n.longValue());
             } else if (type == Float.class) {
@@ -289,7 +294,7 @@ public class DefaultTypeTransformation {
             }
         } else if (type.isPrimitive()) {
             if (type == boolean.class) {
-               return box(booleanUnbox(object));
+               return box(booleanUnbox(object)); 
             } else if (type == byte.class) {
                 return box(byteUnbox(object));
             } else if (type == char.class) {
@@ -415,6 +420,10 @@ public class DefaultTypeTransformation {
         }
         return array;
     }
+    
+    public static <T> Collection<T> asCollection(T[] value) {
+        return arrayAsCollection(value);
+    }
 
     public static Collection asCollection(Object value) {
         if (value == null) {
@@ -464,7 +473,11 @@ public class DefaultTypeTransformation {
         if (value.getClass().getComponentType().isPrimitive()) {
             return primitiveArrayToList(value);
         }
-        return Arrays.asList((Object[]) value);
+        return arrayAsCollection((Object[]) value);
+    }
+
+    public static <T> Collection<T> arrayAsCollection(T[] value) {
+        return Arrays.asList((T[]) value);
     }
 
     /**
@@ -488,7 +501,7 @@ public class DefaultTypeTransformation {
 
         return false;
     }
-
+    
     /**
      * Allows conversion of arrays into a mutable List
      *
@@ -507,7 +520,7 @@ public class DefaultTypeTransformation {
         }
         return list;
     }
-
+    
     public static Object[] primitiveArrayBox(Object array) {
         int size = Array.getLength(array);
         Object[] ret = (Object[]) Array.newInstance(ReflectionCache.autoboxType(array.getClass().getComponentType()), size);
@@ -516,7 +529,7 @@ public class DefaultTypeTransformation {
         }
         return ret;
     }
-
+    
     /**
      * Compares the two objects handling nulls gracefully and performing numeric type coercion if required
      */
@@ -537,7 +550,7 @@ public class DefaultTypeTransformation {
         if (left instanceof Comparable) {
             if (left instanceof Number) {
                 if (isValidCharacterString(right)) {
-                    return castToChar(left) - castToChar(right);
+                    return DefaultGroovyMethods.compareTo((Number) left, (Character) box(castToChar(right)));
                 }
                 if (right instanceof Character || right instanceof Number) {
                     return DefaultGroovyMethods.compareTo((Number) left, castToNumber(right));
@@ -545,15 +558,15 @@ public class DefaultTypeTransformation {
             }
             else if (left instanceof Character) {
                 if (isValidCharacterString(right)) {
-                    return castToChar(left) - castToChar(right);
+                    return DefaultGroovyMethods.compareTo((Character)left,(Character)box(castToChar(right)));
                 }
                 if (right instanceof Number) {
-                    return castToChar(left) - castToChar(right);
+                    return DefaultGroovyMethods.compareTo((Character)left,(Number)right);
                 }
             }
             else if (right instanceof Number) {
                 if (isValidCharacterString(left)) {
-                    return castToChar(left) - castToChar(right);
+                    return DefaultGroovyMethods.compareTo((Character)box(castToChar(left)),(Number) right);
                 }
             }
             else if (left instanceof String && right instanceof Character) {
@@ -662,16 +675,13 @@ public class DefaultTypeTransformation {
         boolean[] ans = null;
 
         // conservative coding
-        if (a.getClass().getName().equals("[Z")) {
+        if (a instanceof boolean[]) {
             ans = (boolean[]) a;
-        }
-        else {
+        } else {
             Object[] ia = (Object[]) a;
             ans = new boolean[ia.length];
             for (int i = 0; i < ia.length; i++) {
-                if (ia[i] == null) {
-                    continue;
-                }
+                if (ia[i] == null) continue;
                 ans[i] = ((Boolean) ia[i]).booleanValue();
             }
         }
@@ -682,10 +692,9 @@ public class DefaultTypeTransformation {
         byte[] ans = null;
 
         // conservative coding
-        if (a.getClass().getName().equals("[B")) {
+        if (a instanceof byte[]) {
             ans = (byte[]) a;
-        }
-        else {
+        } else {
             Object[] ia = (Object[]) a;
             ans = new byte[ia.length];
             for (int i = 0; i < ia.length; i++) {
@@ -701,10 +710,9 @@ public class DefaultTypeTransformation {
         short[] ans = null;
 
         // conservative coding
-        if (a.getClass().getName().equals("[S")) {
+        if (a instanceof short[]) {
             ans = (short[]) a;
-        }
-        else {
+        } else {
             Object[] ia = (Object[]) a;
             ans = new short[ia.length];
             for (int i = 0; i < ia.length; i++) {
@@ -718,10 +726,9 @@ public class DefaultTypeTransformation {
         char[] ans = null;
 
         // conservative coding
-        if (a.getClass().getName().equals("[C")) {
+        if (a instanceof char[]) {
             ans = (char[]) a;
-        }
-        else {
+        } else {
             Object[] ia = (Object[]) a;
             ans = new char[ia.length];
             for (int i = 0; i < ia.length; i++) {
@@ -738,10 +745,9 @@ public class DefaultTypeTransformation {
         long[] ans = null;
 
         // conservative coding
-        if (a.getClass().getName().equals("[J")) {
+        if (a instanceof long[]) {
             ans = (long[]) a;
-        }
-        else {
+        } else {
             Object[] ia = (Object[]) a;
             ans = new long[ia.length];
             for (int i = 0; i < ia.length; i++) {
@@ -758,10 +764,9 @@ public class DefaultTypeTransformation {
         float[] ans = null;
 
         // conservative coding
-        if (a.getClass().getName().equals("[F")) {
+        if (a instanceof float[]) {
             ans = (float[]) a;
-        }
-        else {
+        } else {
             Object[] ia = (Object[]) a;
             ans = new float[ia.length];
             for (int i = 0; i < ia.length; i++) {
@@ -778,10 +783,9 @@ public class DefaultTypeTransformation {
         double[] ans = null;
 
         // conservative coding
-        if (a.getClass().getName().equals("[D")) {
+        if (a instanceof double[]) {
             ans = (double[]) a;
-        }
-        else {
+        } else {
             Object[] ia = (Object[]) a;
             ans = new double[ia.length];
             for (int i = 0; i < ia.length; i++) {
@@ -824,4 +828,14 @@ public class DefaultTypeTransformation {
         }
     }
 
+    public static Character getCharFromSizeOneString(Object value) {
+    	if (value instanceof GString) value = value.toString();
+        if (value instanceof String) {
+            String s = (String) value;
+            if (s.length() != 1) throw new IllegalArgumentException("String of length 1 expected but got a bigger one");
+            return new Character(s.charAt(0));
+        } else {
+            return ((Character) value);
+        }    	
+    }
 }
