@@ -15,8 +15,6 @@
  */
 package org.codehaus.groovy.tools;
 
-import groovy.text.RegexUtils;
-
 import java.io.*;
 import java.net.MalformedURLException;
 import java.net.URL;
@@ -67,7 +65,7 @@ import java.util.regex.Pattern;
  * command does not exist, the path will be ignored.
  *
  * @author Jochen Theodorou
- * @version $Revision: 12828 $
+ * @version $Revision: 14870 $
  * @see RootLoader
  */
 public class LoaderConfiguration {
@@ -183,6 +181,7 @@ public class LoaderConfiguration {
      */
     private void loadFilteredPath(String filter) {
         if (filter == null) return;
+        filter = getSlashyPath(filter);
         int starIndex = filter.indexOf(WILDCARD);
         if (starIndex == -1) {
             addFile(new File(filter));
@@ -190,10 +189,13 @@ public class LoaderConfiguration {
         }
         boolean recursive = filter.indexOf(ALL_WILDCARD) != -1;
 
+        if (filter.lastIndexOf('/')<starIndex) {
+            starIndex=filter.lastIndexOf('/')+1;
+        }
         String startDir = filter.substring(0, starIndex - 1);
         File root = new File(startDir);
 
-        filter = RegexUtils.quote(filter);
+        filter = Pattern.quote(filter);
         filter = filter.replaceAll("\\" + WILDCARD + "\\" + WILDCARD, MATCH_ALL);
         filter = filter.replaceAll("\\" + WILDCARD, MATCH_FILE_NAME);
         Pattern pattern = Pattern.compile(filter);
