@@ -79,8 +79,8 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     private static final Token ASSIGN = Token.newSymbol(Types.ASSIGN, -1, -1);
 
     public void visit(ASTNode[] nodes, SourceUnit source) {
-        if (!(nodes[0] instanceof AnnotationNode) || !(nodes[1] instanceof AnnotatedNode)) {
-            throw new RuntimeException("Internal error: wrong types: $node.class / $parent.class");
+        if (nodes.length != 2 || !(nodes[0] instanceof AnnotationNode) || !(nodes[1] instanceof AnnotatedNode)) {
+            throw new RuntimeException("Internal error: expecting [AnnotationNode, AnnotatedNode] but got: " + Arrays.asList(nodes));
         }
 
         AnnotatedNode parent = (AnnotatedNode) nodes[1];
@@ -95,7 +95,7 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
                 throw new RuntimeException("Error processing interface '" + cName + "'. " + MY_TYPE_NAME + " not allowed for interfaces.");
             }
             if ((cNode.getModifiers() & ACC_FINAL) == 0) {
-                throw new RuntimeException("Error processing class '" + cName + "'. " + MY_TYPE_NAME + " classes must be final.");
+                cNode.setModifiers(cNode.getModifiers() | ACC_FINAL);
             }
 
             final List<PropertyNode> pList = cNode.getProperties();
@@ -227,7 +227,7 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
 
     private Statement returnFalseIfWrongType(ClassNode cNode, Expression other) {
         return new IfStatement(
-                identicalExpr(new ClassExpression(cNode), new ClassExpression(other.getType())),
+                notEqualClasses(cNode, other),
                 new ReturnStatement(ConstantExpression.FALSE),
                 new EmptyStatement()
         );
@@ -507,6 +507,11 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
         return new BooleanExpression(new BinaryExpression(self, COMPARE_IDENTICAL, other));
     }
 
+    private BooleanExpression notEqualClasses(ClassNode cNode, Expression other) {
+        return new BooleanExpression(new BinaryExpression(new ClassExpression(cNode), COMPARE_NOT_EQUAL,
+                new MethodCallExpression(other, "getClass", MethodCallExpression.NO_ARGUMENTS)));
+    }
+
     private Expression findArg(String fName) {
         return new PropertyExpression(new VariableExpression("args"), fName);
     }
