@@ -8,7 +8,6 @@ import org.codehaus.groovy.antlr.GroovySourceASTTest;
 import org.codehaus.groovy.antlr.SourceBufferTest;
 import org.codehaus.groovy.antlr.treewalker.*;
 import org.codehaus.groovy.ast.ClassNodeTest;
-import org.codehaus.groovy.ast.LineColumnCheckTestSuite;
 import org.codehaus.groovy.ast.MethodCallExpressionTest;
 import org.codehaus.groovy.ast.MethodNodeTest;
 import org.codehaus.groovy.ast.ModuleNodeTest;
@@ -90,7 +89,6 @@ public class JavaSourceCodehausPackagesSuite {
         suite.addTestSuite(CompilerConfigurationTest.class);
         suite.addTestSuite(MethodNodeTest.class);
         suite.addTestSuite(VariableExpressionTest.class);
-        suite.addTest(LineColumnCheckTestSuite.suite());
         suite.addTestSuite(MethodCallExpressionTest.class);
         return suite;
     }
