@@ -1,15 +1,7 @@
-import groovy.inspect.InspectorTest;
-import groovy.lang.*;
-import groovy.servlet.GroovyServletTest;
-import groovy.text.TemplateTest;
-import groovy.text.XmlTemplateEngineTest;
-import groovy.tree.NodePrinterTest;
-import groovy.util.EvalTest;
-import groovy.util.MBeanTest;
-import groovy.xml.FactorySupportTest;
 import groovy.xml.XmlTest;
 import junit.framework.Test;
 import junit.framework.TestSuite;
+import org.codehaus.groovy.ast.vm5.LineColumnCheckTestSuite;
 
 /**
  * JDK 1.5 non-security related Java Unit tests in the 'groovy' dir
@@ -20,6 +12,7 @@ public class JavaSourceGroovyPackagesNonSecuritySuite_VM5 {
     public static Test suite() {
         TestSuite suite = new TestSuite();
         suite.addTestSuite(XmlTest.class);
+        suite.addTest(LineColumnCheckTestSuite.suite());
         return suite;
     }
 }
\ No newline at end of file
