@@ -1,5 +1,5 @@
 /*
- $Id: RunGroovyTest.java 11840 2008-04-19 22:58:13Z paulk $
+ $Id: RunGroovyTest.java 14571 2008-12-21 15:50:51Z jimwhite $
 
  Copyright 2003 (C) James Strachan and Bob Mcwhirter. All Rights Reserved.
 
@@ -48,11 +48,13 @@ package org.codehaus.groovy.classgen;
 
 import groovy.lang.GroovyObject;
 
+import java.awt.HeadlessException;
+
 /**
  * Tests dynamically compiling and running a new class
  *
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @version $Revision: 11840 $
+ * @version $Revision: 14571 $
  */
 public class RunGroovyTest extends TestSupport {
 
@@ -92,6 +94,8 @@ public class RunGroovyTest extends TestSupport {
         try {
             GroovyObject object = compile("src/main/groovy/ui/Console.groovy");
         } catch (NoClassDefFoundError e) {
+            
+        } catch (HeadlessException he) {
             // ignore to deal with headless environments
         }
     }
